/*
   Copyright (C) 2003 Clozure Associates
   This file is part of OpenMCL.  

   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
   License , known as the LLGPL and distributed with OpenMCL as the
   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
   which is distributed with OpenMCL as the file "LGPL".  Where these
   conflict, the preamble takes precedence.  

   OpenMCL is referenced in the preamble as the "LIBRARY."

   The LLGPL is also available online at
   http://opensource.franz.com/preamble.html

  $Log: altappservices.c,v $
  Revision 1.2  2003/11/17 07:30:39  gb
  update copyright/license

  Revision 1.1.1.1  2003/11/17 07:14:42  gb
  initial checkin

*/

#include <Carbon/Carbon.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "altappservices.h"


int
installAltConsole(unsigned backend) {
  CFURLRef appURL, exeURL;
  CFBundleRef ourBundle, appBundle;
  CFStringRef appname;
  int fds[2];

  switch (backend) {
  case ALT_CONSOLE_BACKEND_GUI:
    appname = CFSTR("AltConsole");
    break;
    
  default:
    errno = EINVAL;
    return -1;
  }
    
  /* Find this framework's bundle in the file system, then find 
     the indicated application bundle in our "Resources" directory.
     If that succeeds, do a forkpty and have the child execute the
     application bundle's executable file.  Note that it's important
     to use some flavor of exec() to execute the file, since Launch
     and CFBundle services will want to muck around with standard
     I/O.
  */
  if ((ourBundle = 
       CFBundleGetBundleWithIdentifier(CFSTR("com.clozure.altappservices"))) 
      != NULL) {
    if ((appURL = CFBundleCopyResourceURL(ourBundle,
					 appname,
					 CFSTR("app"),
					 NULL)) != NULL) {
      if ((appBundle = CFBundleCreate(NULL, appURL)) != NULL) {
	if ((exeURL = CFBundleCopyExecutableURL(appBundle)) != NULL) {
	  UInt8 path[PATH_MAX];
	  if (CFURLGetFileSystemRepresentation(exeURL,
					       true,
					       path,
					       PATH_MAX)) {
	    char *argv[2], *envp[1];
	    pid_t child_pid;

	    CFRelease(exeURL);
	    CFRelease(appBundle);
	    CFRelease(appURL);
	    CFRelease(ourBundle);
	    
	    socketpair(AF_UNIX, SOCK_STREAM, 0, fds);
	    child_pid = fork();
	    
	    switch(child_pid) {
	    case -1:
	      /* error */
	      close(fds[0]);
	      close(fds[1]);
	      return(-1);

	    default:
	      /* fork successful, this is the parent */
	      close(fds[1]);
	      return fds[0];
	    case 0:
	      /* fork successful, this is the child */
	      close(fds[0]);
	      dup2(fds[1],0);
	      dup2(fds[1],1);
	      dup2(fds[1],2);
	      if (fds[1] > 2) {
		close(fds[1]);
	      }
	      argv[0] = path;
	      argv[1] = NULL;
	      envp[0] = NULL;
	      execve(path,argv,envp);
	      /* should never return */
	      exit(-1);
	      break;
	    }
	  }
	  CFRelease(exeURL);
	}
	CFRelease(appBundle);
      }
      CFRelease(appURL);
    }
    CFRelease(ourBundle);
  }
  return -1;
}

	      
